from typing import Callable, List


class CommandLineInterface:
    """
    This class pretends to be a facilitator to create command line interfaces.
    """

    def __init__(self):
        import argparse
        self.parser = argparse.ArgumentParser()
        self.subscribers = {}

    def add_action(self, name: str, aliases: List[str], action: Callable, type=None, **kwargs):
        """

        Args:
            name: Name of the action
            aliases: Aliases that will be usable
            action: function that will be called if the command line options is called.
            type: type of the argument expected. If None, a bool will be used.
            **kwargs: The kwargs will be directly passed to the parser.add_argument method.

        Returns:
            None
        """
        if type is None:
            self.parser.add_argument(*aliases, dest=name, action='store_true', **kwargs)
        else:
            self.parser.add_argument(*aliases, dest=name, type=type, **kwargs)
        self.subscribers[name] = action

    def parse(self, args=None):
        """
        Parse arguments and trigger
        Args:
            args: Command line arguments. Using stdin if none provided to the call.

        Returns:
            parsed arguments object.
        """
        args = self.parser.parse_args(args)
        self.run_subscribers(args)
        return args

    def run_subscribers(self, args) -> None:
        """
        Goes through the list of arguments and executes the corresponding actoins.
        Args:
            args:

        Returns:
            None
        """
        for name in self.subscribers.keys():
            param = getattr(args, name)
            if param:
                self.subscribers[name](param)


def test():
    """
    A dummy test to see how the class works.
    Returns:

    """

    class DummyClass:
        def __init__(self):
            self.cli = CommandLineInterface()
            self.set_up_cli()

        def set_up_cli(self):
            self.cli.add_action(name="list",
                                aliases=["--list", "-l"],
                                action=self.dummy,
                                )

            self.cli.add_action(name="integer",
                                aliases=["--integer", "-i"],
                                action=self.dummy_retrieve,
                                help="Integer action.",
                                type=int,
                                )
            self.cli.add_action(name="float",
                                aliases=["--float", "-f"],
                                action=self.dummy_retrieve,
                                help="Float action.",
                                type=float,
                                )

        def dummy(self, *args, **kwargs):
            print(f"Dummy called from {self}")
            print("Yummy dummy!")

        @staticmethod
        def dummy_retrieve(dummy_id):
            print(f"Received {type(dummy_id)}: {dummy_id}")

        def run(self, args=None):
            return self.cli.parse(args)

    dummy = DummyClass()

    cases = [
        ["--integer", "1"],
        ["--float", "1.2"],
        ["-l"],
        None,
    ]
    for case in cases:
        dummy.run(case)

